#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_scheduler.interfaces.zodb import IZODBPackingTask
from pyams_utils.interfaces.zeo import IZEOConnection
from zope.component.interfaces import ISite

# import packages
from pyams_scheduler.task import Task
from pyams_utils.traversing import get_parent
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


@implementer(IZODBPackingTask)
class ZODBPackingTask(Task):
    """ZODB packing task"""

    zeo_connection = FieldProperty(IZODBPackingTask['zeo_connection'])
    pack_time = FieldProperty(IZODBPackingTask['pack_time'])

    def run(self, report):
        site = get_parent(self, ISite)
        sm = site.getSiteManager()
        zeo_connection = sm.queryUtility(IZEOConnection, self.zeo_connection)
        if zeo_connection is None:
            report.write("Can't find ZEO connection. Task aborted.")
            return
        report.write("Loaded ZEO connection {0}\n".format(self.zeo_connection))
        report.write("Packing transactions older than {0} days\n".format(self.pack_time))
        storage, db = zeo_connection.get_connection(get_storage=True)
        try:
            db.pack(days=self.pack_time)
            report.write("\nPack successful.\n")
        finally:
            storage.close()
